--[[
	* Copyright (c) 2019 by Antoine Desmarets.
	* Cixi of Remulos Oceanic / WoW Classic
	*
	* Geared is distributed in the hope that it will be useful and entertaining,
	* but WITHOUT ANY WARRANTY
]]--


local gearedlocal_version = "1.13.6.002"  --change here, and in TOC
local gearedlocal_reqPrefix = "Geared;"

local Geared_button_tr = nil
local Geared_button_cr = nil

local Geared_frame = CreateFrame("Frame")
Geared_frame:RegisterEvent("ADDON_LOADED")				--Initialisation
Geared_frame:RegisterEvent("PLAYER_LOGIN")				--Player in world


Geared_frame:SetScript("OnEvent", function(self, event, ...)

	local arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg, arg9 = ...
	--print(event)
	if event == "ADDON_LOADED" and arg1 == "Geared" then
		DEFAULT_CHAT_FRAME:AddMessage("|cffff00ff[Geared]|r v."..gearedlocal_version.." by Cixi@Remulos. Open a profession window to get started,")
	end

	if event == "PLAYER_LOGIN" then 
		if Geared_button_tr == nil then 
			Geared_button_tr = CreateFrame("Button","CharacterExport",PaperDollItemsFrame) 
			Geared_button_tr:SetWidth(48)
			Geared_button_tr:SetHeight(48)
			Geared_button_tr:SetPoint("TOPLEFT", PaperDollItemsFrame, "TOPRIGHT", -30, -13)
			Geared_button_tr:SetNormalTexture("Interface/Icons/inv_helmet_49")
			Geared_button_tr:SetHighlightTexture("Interface/Icons/inv_helmet_49")
			Geared_button_tr:SetToplevel(true)
			Geared_button_tr:SetScript("OnClick", function() 
				Geared_Export(true) 
			end)
		end
	end

end)

-------------------------------------------------------------------------

function Geared_round(num, numDecimalPlaces)
  local mult = 10^(numDecimalPlaces or 0)
  return math.floor(num * mult + 0.5) / mult
end

-------------------------------------------------------------------------

function Geared_Export(isTr)


	local player = {}
	player.n = UnitName("player")
	player.R = GetRealmName()
	player.l = UnitLevel("player")
	player.F = UnitFactionGroup("player")
	_, classFile = UnitClass("player")
	_, raceFile = UnitRace("player")

	local g = UnitSex("player")
	player.g = 'male'
	if g == 3 then player.g = 'female' end
	player.c = classFile
	player.r = raceFile   --Scourge, Troll, etc
	
	local serplayer = Geared_serialize(player)
	local ser = gearedlocal_version .. "##" .. serplayer .. "##"

	ser = ser .. 'health:' .. UnitHealthMax("player") .."|";
	ser = ser .. 'mana:' .. UnitPowerMax("player") .."|";

	for i = 1, GetNumTalentTabs() do
		local cnt = 0;
		for j = 1, GetNumTalents(i) do
			name, iconTexture, tier, column, rank, maxRank = GetTalentInfo(i, j);
			cnt = cnt + rank;
		end
		ser = ser .. 'talenttab' .. i .. ':' ..cnt .."|";	
	
	end


	for i=1, GetNumSkillLines() do
	--print (GetSkillLineInfo(i));
		local name, _, _, skill, numPoints, mod = GetSkillLineInfo(i)
		skill = skill + numPoints + mod;
		if skill > 0 then ser = ser .. name .. ':' .. skill .."|"; end
		--print(name, skill);
		if name == DEFENSE then ser = ser .. 'def:' .. skill .."|"; end
	end
	
	
	local buffs = "";
	for i=1, 40 do
	--print (GetSkillLineInfo(i));
		name, _, _, _, _, _, _, _, _, spellId = UnitAura("player", i);
		if (name ~= nil) then buffs = buffs .. spellId .. ";"; end
	end
	buffs = string.sub(buffs, 1, string.len(buffs)-1) -- remove last separator
	ser = ser .. 'buffs:' .. buffs .."|";
		
	ser = ser .. 'str:' .. UnitStat("player", 1) .."|";
	ser = ser .. 'agi:' .. UnitStat("player", 2) .."|";
	ser = ser .. 'sta:' .. UnitStat("player", 3) .."|";
	ser = ser .. 'int:' .. UnitStat("player", 4) .."|";
	ser = ser .. 'spi:' .. UnitStat("player", 5) .."|";
	
	_, total = UnitArmor("player"); 	ser = ser .. 'armor:' .. total .."|";
	
	
	local  baseDefense, armorDefense = UnitDefense("player");
	--total = baseDefense + armorDefense;	ser = ser .. 'def:' .. total .."|";
	ser = ser .. 'parrypct:' .. Geared_round(GetParryChance(), 2) .."|";
	ser = ser .. 'dodgepct:' .. Geared_round(GetDodgeChance(), 2) .."|";
	ser = ser .. 'blockpct:' .. Geared_round(GetBlockChance(), 2) .."|";

	ser = ser .. 'manargn:' .. Geared_round(GetManaRegen(), 2) .."|";

	base, posBuff, negBuff = UnitAttackPower("player"); 	
	total = base + posBuff + negBuff;	ser = ser .. 'atkpwr:' .. total .."|";
	ser = ser .. 'mlecritstrkpct:' .. Geared_round(GetCritChance("player"), 2) .."|";
	ser = ser .. 'mlehitpct:' .. GetHitModifier() .."|";

	ser = ser .. 'rgdatkpwr:' .. UnitRangedAttackPower("player") .."|";
	ser = ser .. 'rgdcritstrkpct:' .. Geared_round(GetRangedCritChance("player"), 2) .."|";
	ser = ser .. 'rgdhitpct:' .. GetHitModifier() .."|";

	_, total = UnitResistance("player", 6);		ser = ser .. 'arcresist:' .. total .."|";
	_, total = UnitResistance("player", 2);		ser = ser .. 'firresist:' .. total .."|";
	_, total = UnitResistance("player", 3);		ser = ser .. 'natresist:' .. total .."|";
	_, total = UnitResistance("player", 4);		ser = ser .. 'froresist:' .. total .."|";
	_, total = UnitResistance("player", 5);		ser = ser .. 'sharesist:' .. total .."|";	
	
	ser = ser .. 'arcsplpwr:'.. GetSpellBonusDamage(7) .. "|" --arcane dmg
	ser = ser .. 'firsplpwr:'.. GetSpellBonusDamage(3) .. "|" --fire dmg
	ser = ser .. 'natsplpwr:'.. GetSpellBonusDamage(4) .. "|" --nature dmg
	ser = ser .. 'frosplpwr:'.. GetSpellBonusDamage(5) .. "|" --frost dmg
	ser = ser .. 'shasplpwr:'.. GetSpellBonusDamage(6) .. "|" --shadow dmg
	ser = ser .. 'splheal:' .. GetSpellBonusHealing() .."|"
	ser = ser .. 'splpwr:'.. GetSpellBonusDamage(1) .. "|" --spell dmg
	
	ser = ser .. 'splcritstrkpct:' .. Geared_round(GetSpellCritChance("player"), 2) .."|";
	ser = ser .. 'splhitpct:' .. GetSpellHitModifier() .."|";
		
	
	ser = string.sub(ser, 1, string.len(ser)-1) -- remove last ppipe
	--print(ser);
	ser = ser .. "##"

	for i=1, 19 do 
		local itemLink = GetInventoryItemLink("player", i)
		if itemLink == nil then 
			--ser = ser .. '\n'.. i .. ': ' .. '(empty)' --equipped slots
		else
			--local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture, itemSellPrice = GetItemInfo(itemLink) 
			local _, _, Colour, Ltype, Id = string.find(itemLink, "|?c?f?f?(%x*)|?H?([^:]*):?(%d+):?(%d*):?(%d*):?(%d*):?(%d*):?(%d*):?(%-?%d*):?(%-?%d*):?(%d*):?(%d*):?(%-?%d*)|?h?%[?([^%[%]]*)%]?|?h?|?r?")
			ser = ser .. i .. ':' .. Id .. "|" --equipped slots
		end
		
	end
        
	ser = string.sub(ser, 1, string.len(ser)-1) -- remove last ppipe

	local encoded = Geared_enc(ser)
	
	StaticPopupDialogs["EXPORT_GEARED"] = {
		text = "Copy the text below, then upload it to\n\nhttp://warcraftratings.com/geared/upload",
		button1 = "Done",
		OnShow = function (self, data)
			self.editBox:SetText(""..encoded)
			self.editBox:HighlightText()
			self.editBox:SetScript("OnEscapePressed", function(self) StaticPopup_Hide ("EXPORT_GEARED") end)
		end,
		timeout = 0,
		hasEditBox = true,
		editBoxWidth = 350,
		whileDead = true,
		hideOnEscape = 1,
		preferredIndex = 3,  -- avoid some UI taint, see http://www.wowace.com/announcements/how-to-avoid-some-ui-taint/
	}
	StaticPopup_Show ("EXPORT_GEARED")


end

-------------------------------------------------------------------------

function Geared_serialize (o)
	local res = ""
	if type(o) == "number" then
	  res = res .. o
	elseif type(o) == "string" then
		res = res .. string.format("%q", o)
	elseif type(o) == "table" then
		res = res .. "{\n"
	  for k,v in pairs(o) do
		if type(k) == "number" then
			res = res .. "  [" .. k .. "] = "
		elseif type(k) == "string" then
			 res = res .. "  [\"" .. k .. "\"] = "
		end
		res = res .. Geared_serialize(v)
		res = res .. ",\n"
	  end
	  res = res ..  "}"
	elseif type(o) == "boolean" then
		if o then res = res .. "true"
		else res = res .. "false" end
	else
	  error("cannot serialize a " .. type(o))
	end
	return res
  end

-------------------------------------------------------------------------

local b='ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/' -- You will need this for encoding/decoding
-- encoding
function Geared_enc(data)
    return ((data:gsub('.', function(x) 
        local r,b='',x:byte()
        for i=8,1,-1 do r=r..(b%2^i-b%2^(i-1)>0 and '1' or '0') end
        return r;
    end)..'0000'):gsub('%d%d%d?%d?%d?%d?', function(x)
        if (#x < 6) then return '' end
        local c=0
        for i=1,6 do c=c+(x:sub(i,i)=='1' and 2^(6-i) or 0) end
        return b:sub(c+1,c+1)
    end)..({ '', '==', '=' })[#data%3+1])
end

-------------------------------------------------------------------------
-- decoding
function Geared_dec(data)
    data = string.gsub(data, '[^'..b..'=]', '')
    return (data:gsub('.', function(x)
        if (x == '=') then return '' end
        local r,f='',(b:find(x)-1)
        for i=6,1,-1 do r=r..(f%2^i-f%2^(i-1)>0 and '1' or '0') end
        return r;
    end):gsub('%d%d%d?%d?%d?%d?%d?%d?', function(x)
		if (#x ~= 8) then return '' end
        local c=0
        for i=1,8 do c=c+(x:sub(i,i)=='1' and 2^(8-i) or 0) end
            return string.char(c)
    end))
end
